require 'coding_ar_source'
require 'coding_ar_command'
require 'coding_ar_util'

module Pod
  module Downloader
    class Http
      def self.options
        [:type, :flatten, :sha1, :sha256, :indexDownload]
      end

      alias_method :orig_download_file, :download_file
      alias_method :orig_should_flatten?, :should_flatten?

      def download_file(full_filename)
        CodingArUtil.download(url, full_filename)
      end

      def should_flatten?
        if options.key?(:indexDownload)
          true
        else
          orig_should_flatten?
        end
      end
    end
  end
end

module Pod
  class Source
    class Manager
      alias_method :orig_source_from_path, :source_from_path
      alias_method :orig_create_source_with_url, :create_source_with_url

      def source_from_path(path)
        @sources_by_path ||= Hash.new do |hash, key|
          hash[key] = case
                      when key.basename.to_s == Pod::TrunkSource::TRUNK_REPO_NAME
                        TrunkSource.new(key)
                      when (key + '.url').exist?
                        CDNSource.new(key)
                      when (key + '.coding_ar_url').exist?
                        CodingArSource.new(key)
                      else
                        Source.new(key)
                      end
        end

        @sources_by_path[path]
      end

      def create_source_with_url(url)
        require 'coding_ar_util'
        
        name = name_for_url(url)
        if CodingArUtil.coding_ar_service?(url)
          Command::Repo::AddCodingAr.parse([name, url]).run
        else
          orig_create_source_with_url(url)
        end

        source = source_with_url(url)
        raise "Unable to create a source with URL #{url}" unless source
        source
      end
    end
  end
end