require 'json'
require 'typhoeus'
require 'netrc'
require 'cocoapods_coding_ar'

module Pod
  class CodingArUtil
    CODING_AR_NETRC_PATH_ENV = 'COCOAPODS_CODING_AR_NETRC_PATH'
    JSON_CONTENT_TYPE = 'application/json; charset=utf-8'
    USER_AGENT = "cocoapods-coding-ar/#{CocoapodsCodingAr::VERSION} cocoapods/#{Pod::VERSION} #{RUBY_ENGINE}/#{RUBY_VERSION}-p#{RUBY_PATCHLEVEL} (#{IO.popen('uname -v').gets.strip})"

    def self.coding_ar_service?(url)
      request = Typhoeus::Request.new(
        url,
        :method => :get,
        :headers => { 
          'User-Agent' => USER_AGENT, 
        },
      )
      request.run

      begin
        resp = JSON.parse(request.response.body)
        resp['service'] == 'CODING-AR'
      rescue
        return false
      end
    end

    def self.push_pod(url, body_path)
      request = Typhoeus::Request.new(
        url,
        :method => :post,
        :netrc => :optional,
        :netrc_file => ENV[CODING_AR_NETRC_PATH_ENV] || Netrc.default_path,
        :headers => { 
          'Content-Type' => JSON_CONTENT_TYPE,
          'User-Agent' => USER_AGENT, 
        },
        :body => File.read(body_path),
      )
      request.run

      if request.response.code == 0
        raise "fail to push pod: #{request.response.return_message}"
      elsif request.response.code != 200
        raise "fail to push pod: #{request.response.body}"
      end
    end

    def self.get_json(url)
      request = Typhoeus::Request.new(
        url,
        :method => :get,
        :netrc => :optional,
        :netrc_file => ENV[CODING_AR_NETRC_PATH_ENV] || Netrc.default_path,
        :followlocation => true,
        :headers => { 
          'User-Agent' => USER_AGENT, 
        },
      )
      request.run

      if request.response.code == 0
        raise "fail to request #{url}: #{request.response.return_message}"
      elsif request.response.code != 200
        raise "fail to request #{url}: #{request.response.body}"
      else
        JSON.parse(request.response.body)
      end
    end

    def self.download(url, path)
      request = Typhoeus::Request.new(
        url,
        :method => :get,
        :netrc => :optional,
        :netrc_file => ENV[CODING_AR_NETRC_PATH_ENV] || Netrc.default_path,
        :followlocation => true,
        :headers => { 
          'User-Agent' => USER_AGENT, 
        },
      )
      request.run

      if request.response.code == 0
        raise "fail to request #{url}: #{request.response.return_message}"
      elsif request.response.code != 200
        raise "fail to request #{url}: #{request.response.body}"
      else
        file = File.new(path, 'wb')
        file << request.response.body
        file.close
      end
    end

  end
end